/* SPDX-License-Identifier: GPL-2.0-or-later */
/*
 * Copyright (C) 2023 Verisilicon Inc.
 */

#include <linux/uaccess.h>
#include <linux/kernel.h>
#include <linux/version.h>
#include <linux/errno.h>
#include <linux/time.h>
#include <linux/rtc.h>
#include <linux/device.h>
#include <linux/kthread.h>
#include <linux/sched.h>

#include "reg_dump.h"
#include "vc8000d.h"
#include "vc8000e.h"
#include "vcmd/hantrovcmd.h"
#include "hdma.h"

/* dump */
typedef int (*dump_fun)(struct sn_tranx_t *, struct file *);
struct ip_dump_info {
	char name[16];
	dump_fun dump;
	enum TRANS_MODULE_INDEX index;
};

struct reg_dump_t {
	struct mutex reg_dump_lock;
	struct sn_tranx_t *tdev;
};

/* task */
struct task_info {
	struct task_struct *dump_task;
	struct sn_tranx_t *tdev;
	struct mutex task_lock;
	int task_count;
	int task_status;
	char ip_name[16];
};

static struct task_info task_dump = {0};

static struct file *open_dump_file(char* string, mm_segment_t* fs)
{
	struct file *filp = NULL;
	filp = filp_open(string, O_RDWR | O_CREAT | O_APPEND, 0644);
	if (IS_ERR(filp)) {
		return NULL;
	}
#if LINUX_VERSION_CODE < KERNEL_VERSION(5,10,0)
	*fs = get_fs();
	set_fs(KERNEL_DS);
#else
	*fs = force_uaccess_begin();
#endif
	return filp;
}

static int close_dump_file(struct file * filp, mm_segment_t* fs)
{
	if (filp == NULL || fs == NULL)
		return -EFAULT;
#if LINUX_VERSION_CODE < KERNEL_VERSION(5,10,0)
	set_fs(*fs);
#else
	force_uaccess_end(*fs);
#endif
	filp_close(filp, NULL);
	return 0;
}

int write_dump_file(struct file * filp, char* buf, int len)
{
	int ret = 0;
	if (buf == NULL)
		return -EFAULT;

	if (filp == NULL){
		printk("%s", buf);
	} else {
		ret = kernel_write(filp, buf, len, &filp->f_pos);
	}
	memset(buf, 0, len);
	return ret;
}

static struct ip_dump_info ip_dumps[] = {
	{"HDMA", hdma_dump_regs, SN_MODULE_HDMA},
#if SUB_SYS_VCD
	{"VC8000D", vcd_dump_regs, SN_MODULE_VC8000D},
#endif
#if SUB_SYS_VCE
	{"VC8000E", vce_dump_regs, SN_MODULE_VC8000E},
#endif
#if (VCMD_ENABLE_VC8000D == 1 || VCMD_ENABLE_VC8000E == 1)
	{"VCMD", vcmd_dump_regs, SN_MODULE_VCMD}
#endif
};

int dump_ip_regs(struct sn_tranx_t *tdev, char *buf)
{
	char path_name[64];
	struct file *filp = NULL;
	mm_segment_t fs;
	ktime_t k_time;
	struct rtc_time tm;
	struct reg_dump_t *dump = tdev->modules[SN_MODULE_REG_DUMP];
	int ret = 0, i = 0, j = 0;
	if (tdev == NULL || buf == NULL)
		return -EFAULT;
	for (i = 0; i < ARRAY_SIZE(ip_dumps); i++) {
		if (strcasecmp(buf, ip_dumps[i].name) == 0) {
			if (!(tdev->init_flag & (1 << ip_dumps[i].index))) {
				sn_pri(tdev, SN_ERR, "%s: the driver do not load %s\n",
						__func__, ip_dumps[i].name);
				return -ENODEV;
			}
			k_time = ktime_get_real();
			tm = rtc_ktime_to_tm(k_time);
			sprintf(path_name, "/var/log/reg-%s-%s-%02d%02d%02d.log", tdev->dev_name, ip_dumps[i].name, tm.tm_hour + 8, tm.tm_min, tm.tm_sec);
			mutex_lock(&dump->reg_dump_lock);
			filp = open_dump_file(path_name, &fs);
			if (filp == NULL) {
				sn_pri(tdev, SN_ERR, "%s: open file failed.\n", __func__);
				mutex_unlock(&dump->reg_dump_lock);
				return -EFAULT;
			}
			sn_pri(tdev, SN_INF, "The dump log has wrote to : %s\n", path_name);
			ret = ip_dumps[i].dump(tdev, filp);
			if (ret) {
				sn_pri(tdev, SN_ERR, "%s: dump file failed. ret = %d\n", __func__, ret);
				mutex_unlock(&dump->reg_dump_lock);
				return ret;
			}

			ret = close_dump_file(filp, &fs);
			if (ret) {
				sn_pri(tdev, SN_ERR, "%s: close file failed. ret = %d\n", __func__, ret);
				mutex_unlock(&dump->reg_dump_lock);
				return ret;
			}
			mutex_unlock(&dump->reg_dump_lock);
			break;
		}
	}
	if (i >= ARRAY_SIZE(ip_dumps)) {
		sn_pri(tdev, SN_ERR, "%s: the input is %s, it not matches to",
				__func__, buf);
		for (j = 0; j < ARRAY_SIZE(ip_dumps); j++)
			sn_pri(tdev, SN_ERR, "%s", ip_dumps[j].name);
		return -EINVAL;
	}
	else
		return 0;
}

static ssize_t dump_ips_regs_store(struct device *dev, struct device_attribute *attr,
			     const char *buf, size_t count)
{
	struct sn_misc_tdev *mtdev = dev_get_drvdata(dev);
	struct sn_tranx_t *tdev = mtdev->tdev;
	char ip_name_buffer[16] = {0};
	int ret = 0;
	int buffer_offset = 0;

	if (count == 0)
		return 0;

	while (*(buf + buffer_offset) != '\0') {
		if (sscanf(buf + buffer_offset, "%15s", ip_name_buffer) != 1) {
			sn_pri(tdev, SN_ERR, "%s: please input string format\n", __func__);
			return count;
		}
		ret = dump_ip_regs(tdev, ip_name_buffer);
		if (ret) {
			sn_pri(tdev, SN_ERR, "%s: dump_ip_regs failed. ret = %d\n", __func__, ret);
			return count;
		}
		buffer_offset += strlen(ip_name_buffer) + 1;
		memset(ip_name_buffer, 0, sizeof(ip_name_buffer));
	}
	return count;
}

static DEVICE_ATTR_WO(dump_ips_regs);

static struct attribute *trans_dump_sysfs_entries[] = {
	&dev_attr_dump_ips_regs.attr,
	NULL
};

static struct attribute_group trans_dump_attribute_group = {
	.name = NULL,
	.attrs = trans_dump_sysfs_entries,
};

int sn_reg_dump_init(struct sn_tranx_t *tdev)
{
	int ret = 0;
	struct reg_dump_t *dump;
	dump = kzalloc(sizeof(struct reg_dump_t), GFP_KERNEL);
	if (!dump) {
		sn_pri(tdev, SN_ERR, "%s: alloc reg_dump_t failed.\n",
				__func__);
		return -EFAULT;
	}
	tdev->modules[SN_MODULE_REG_DUMP] = dump;
	dump->tdev = tdev;
	mutex_init(&dump->reg_dump_lock);
	ret = sysfs_create_group(&tdev->misc_dev->this_device->kobj,
				&trans_dump_attribute_group);
	if (ret) {
		sn_pri(tdev, SN_ERR,
				"dump: failed to create sysfs device attributes\n");
		mutex_destroy(&dump->reg_dump_lock);
		kfree(dump);
		return -EFAULT;
	}
	sn_pri(tdev, SN_INF, "dump: module initialize done.\n");
	return 0;

}

void sn_reg_dump_release(struct sn_tranx_t *tdev)
{
	struct reg_dump_t *dump = tdev->modules[SN_MODULE_REG_DUMP];
	mutex_destroy(&dump->reg_dump_lock);
	sysfs_remove_group(&tdev->misc_dev->this_device->kobj,
				&trans_dump_attribute_group);
	kfree(dump);
	sn_pri(tdev, SN_DBG, "dump: remove module done.\n");
}

static int kthread_dump_func(void* data)
{
	int ret = 0;
	while (!kthread_should_stop()) {
		switch (task_dump.task_status) {
			case TASK_IDEL:
				msleep(SLEEP_X_MS);
				break;
			case TASK_BUSY:
				if (task_dump.tdev) {
					ret = dump_ip_regs(task_dump.tdev, task_dump.ip_name);
					if (ret)
						sn_pri(task_dump.tdev, SN_ERR, "%s:dump %s has error. ret = %d\n",
								__func__, task_dump.ip_name, ret);
				}
				task_dump.task_status = TASK_IDEL;
				break;
			default:
				break;
		}
	}
	return 0;
}

int create_dump_task(struct sn_tranx_t *tdev)
{
	if (tdev == NULL)
		return -EFAULT;
	if (!task_dump.task_count) {
		task_dump.dump_task = kthread_create(kthread_dump_func, NULL, "kthread-dump");
		if (IS_ERR(task_dump.dump_task)) {
			task_dump.dump_task = NULL;
			task_dump.tdev = NULL;
			return -EFAULT;
		}
		task_dump.tdev = tdev;
		wake_up_process(task_dump.dump_task);
		mutex_init(&task_dump.task_lock);
		task_dump.task_count++;
	}
	else
		task_dump.task_count++;
	return 0;
}

void release_dump_task(void)
{
	if (task_dump.task_count == 1) {
		if (task_dump.dump_task) {
			kthread_stop(task_dump.dump_task);
			task_dump.dump_task = NULL;
			task_dump.tdev = NULL;
		}
		mutex_destroy(&task_dump.task_lock);
		task_dump.task_count = 0;
	}
	else {
		task_dump.task_count--;
	}
}

int task_dump_ips(struct sn_tranx_t *tdev, char* ip_name)
{
	int delay = 0, ret = 0;
	if (tdev == NULL || ip_name == NULL)
		return -EFAULT;
	task_dump.tdev = tdev;
	strlcpy(task_dump.ip_name, ip_name, strlen(ip_name) + 1);
	mutex_lock(&task_dump.task_lock);
	if (task_dump.task_status == TASK_BUSY) {
		sn_pri(tdev, SN_ERR, "%s:task is busy\n", __func__);
		mutex_unlock(&task_dump.task_lock);
		return -EAGAIN;
	}
	task_dump.task_status = TASK_BUSY;
	delay = 6000;
	while (delay--) {
		if (task_dump.task_status == TASK_IDEL)
			break;
		usleep_range(1000, 1020);
	}
	if (task_dump.task_status != TASK_IDEL) {
		sn_pri(tdev, SN_ERR, "%s:dump is timeout\n", __func__);
		ret = -EFAULT;
	}
	mutex_unlock(&task_dump.task_lock);
	return ret;
}
